@@ -1,11 +1,11 @@
 // Copyright (C) 2021 Scott Lamb <slamb@slamb.org>
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
-use bytes::{Buf, Bytes};
+use bytes::Bytes;
 use log::debug;
 use pretty_hex::PrettyHex;
-use sdp::media_description::MediaDescription;
-use std::{convert::TryFrom, net::IpAddr, num::NonZeroU16};
+use sdp_types::Media;
+use std::{net::IpAddr, num::NonZeroU16};
 use url::Url;
 
 use super::{Presentation, Stream};
@@ -224,21 +224,16 @@ pub(crate) fn get_cseq(response: &rtsp_types::Response<Bytes>) -> Option<u32> {
 /// Parses a [MediaDescription] to a [Stream].
 /// On failure, returns an error which is expected to be supplemented with
 /// the [MediaDescription] debug string and packed into a `RtspResponseError`.
-fn parse_media(base_url: &Url, media_description: &MediaDescription) -> Result<Stream, String> {
-    let media = media_description.media_name.media.clone();
+fn parse_media(base_url: &Url, media_description: &Media) -> Result<Stream, String> {
+    let media = media_description.media.clone();
 
     // https://tools.ietf.org/html/rfc8866#section-5.14 says "If the <proto>
     // sub-field is "RTP/AVP" or "RTP/SAVP" the <fmt> sub-fields contain RTP
     // payload type numbers."
     // https://www.iana.org/assignments/sdp-parameters/sdp-parameters.xhtml#sdp-parameters-2
-    // shows several other variants, such as "TCP/RTP/AVP". Looking a "RTP" component
+    // shows several other variants, such as "TCP/RTP/AVP". Looking for a "RTP" component
     // seems appropriate.
-    if !media_description
-        .media_name
-        .protos
-        .iter()
-        .any(|p| p == "RTP")
-    {
+    if !media_description.proto.starts_with("RTP/") && !media_description.proto.contains("/RTP/") {
         return Err("Expected RTP-based proto".into());
     }
 
@@ -248,10 +243,10 @@ fn parse_media(base_url: &Url, media_description: &MediaDescription) -> Result<S
     // format for the session." Just use the first until we find a stream
     // where this isn't the right thing to do.
     let rtp_payload_type_str = media_description
-        .media_name
-        .formats
-        .first()
-        .ok_or_else(|| "missing RTP payload type".to_string())?;
+        .fmt
+        .split_ascii_whitespace()
+        .next()
+        .unwrap();
     let rtp_payload_type = u8::from_str_radix(rtp_payload_type_str, 10)
         .map_err(|_| format!("invalid RTP payload type {:?}", rtp_payload_type_str))?;
     if (rtp_payload_type & 0x80) != 0 {
@@ -268,7 +263,7 @@ fn parse_media(base_url: &Url, media_description: &MediaDescription) -> Result<S
     let mut fmtp = None;
     let mut control = None;
     for a in &media_description.attributes {
-        if a.key == "rtpmap" {
+        if a.attribute == "rtpmap" {
             let v = a
                 .value
                 .as_ref()
@@ -287,7 +282,7 @@ fn parse_media(base_url: &Url, media_description: &MediaDescription) -> Result<S
             if rtpmap_payload_type == rtp_payload_type_str {
                 rtpmap = Some(v);
             }
-        } else if a.key == "fmtp" {
+        } else if a.attribute == "fmtp" {
             // Similarly starts with payload-type SP.
             let v = a
                 .value
@@ -299,7 +294,7 @@ fn parse_media(base_url: &Url, media_description: &MediaDescription) -> Result<S
             if fmtp_payload_type == rtp_payload_type_str {
                 fmtp = Some(v);
             }
-        } else if a.key == "control" {
+        } else if a.attribute == "control" {
             control = a
                 .value
                 .as_deref()
@@ -385,24 +380,13 @@ pub(crate) fn parse_describe(
         ));
     }
 
-    let sdp;
-    {
-        let mut cursor = std::io::Cursor::new(&response.body()[..]);
-        sdp =
-            sdp::session_description::SessionDescription::unmarshal(&mut cursor).map_err(|e| {
-                format!(
-                    "Unable to parse SDP: {}\n\n{:#?}",
-                    e,
-                    response.body().hex_dump()
-                )
-            })?;
-        if cursor.has_remaining() {
-            return Err(format!(
-                "garbage after sdp: {:?}",
-                &response.body()[usize::try_from(cursor.position()).unwrap()..]
-            ));
-        }
-    }
+    let sdp = sdp_types::Session::parse(&response.body()[..]).map_err(|e| {
+        format!(
+            "Unable to parse SDP: {}\n\n{:#?}",
+            e,
+            response.body().hex_dump()
+        )
+    })?;
 
     // https://tools.ietf.org/html/rfc2326#appendix-C.1.1
     let base_url = response
@@ -417,20 +401,23 @@ pub(crate) fn parse_describe(
         .unwrap_or(Ok(request_url.clone()))?;
 
     let mut control = None;
+    let mut tool = None;
     for a in &sdp.attributes {
-        if a.key == "control" {
+        if a.attribute == "control" {
             control = a
                 .value
                 .as_deref()
                 .map(|c| join_control(&base_url, c))
                 .transpose()?;
             break;
+        } else if a.attribute == "tool" {
+            tool = a.value.as_deref().map(Into::into);
         }
     }
     let control = control.unwrap_or(request_url);
 
     let streams = sdp
-        .media_descriptions
+        .medias
         .iter()
         .enumerate()
         .map(|(i, m)| {
@@ -447,11 +434,18 @@ pub(crate) fn parse_describe(
         base_url,
         control,
         accept_dynamic_rate,
+        tool,
     })
 }
 
-pub(crate) struct SetupResponse<'a> {
-    pub(crate) session_id: &'a str,
+#[derive(Debug, PartialEq, Eq)]
+pub(crate) struct SessionHeader {
+    pub(crate) id: Box<str>,
+    pub(crate) timeout_sec: u32,
+}
+
+pub(crate) struct SetupResponse {
+    pub(crate) session: SessionHeader,
     pub(crate) ssrc: Option<u32>,
     pub(crate) channel_id: Option<u8>,
     pub(crate) source: Option<IpAddr>,
@@ -463,12 +457,27 @@ pub(crate) struct SetupResponse<'a> {
 /// Returns an assigned interleaved channel id (implying the next channel id
 /// is also assigned) or errors.
 pub(crate) fn parse_setup(response: &rtsp_types::Response<Bytes>) -> Result<SetupResponse, String> {
+    // https://datatracker.ietf.org/doc/html/rfc2326#section-12.37
     let session = response
         .header(&rtsp_types::headers::SESSION)
         .ok_or_else(|| "Missing Session header".to_string())?;
-    let session_id = match session.as_str().find(';') {
-        None => session.as_str(),
-        Some(i) => &session.as_str()[..i],
+    let session = match session.as_str().split_once(';') {
+        None => SessionHeader {
+            id: session.as_str().into(),
+            timeout_sec: 60, // default
+        },
+        Some((id, timeout_str)) => {
+            if let Some(v) = timeout_str.trim().strip_prefix("timeout=") {
+                let timeout_sec =
+                    u32::from_str_radix(v, 10).map_err(|_| format!("Unparseable timeout {}", v))?;
+                SessionHeader {
+                    id: id.into(),
+                    timeout_sec,
+                }
+            } else {
+                return Err(format!("Unparseable Session header {:?}", session.as_str()));
+            }
+        }
     };
     let transport = response
         .header(&rtsp_types::headers::TRANSPORT)
@@ -516,7 +525,7 @@ pub(crate) fn parse_setup(response: &rtsp_types::Response<Bytes>) -> Result<Setu
         }
     }
     Ok(SetupResponse {
-        session_id,
+        session,
         ssrc,
         channel_id,
         source,
@@ -612,11 +621,13 @@ pub(crate) fn parse_play(
 mod tests {
     use std::num::NonZeroU16;
 
+    use bytes::Bytes;
     use url::Url;
 
     use crate::{client::StreamStateInit, codec::Parameters};
 
     use super::super::StreamState;
+    use super::SessionHeader;
     use crate::testutil::response;
 
     fn parse_describe(
@@ -627,6 +638,30 @@ mod tests {
         super::parse_describe(url, &response(raw_response))
     }
 
+    #[test]
+    fn anvpiz_sdp() {
+        let url = Url::parse("rtsp://127.0.0.1/").unwrap();
+        let response =
+            rtsp_types::Response::builder(rtsp_types::Version::V1_0, rtsp_types::StatusCode::Ok)
+                .header(rtsp_types::headers::CONTENT_TYPE, "application/sdp")
+                .build(Bytes::from_static(include_bytes!(
+                    "testdata/anpviz_sdp.txt"
+                )));
+        super::parse_describe(url, &response).unwrap();
+    }
+
+    #[test]
+    fn geovision_sdp() {
+        let url = Url::parse("rtsp://127.0.0.1/").unwrap();
+        let response =
+            rtsp_types::Response::builder(rtsp_types::Version::V1_0, rtsp_types::StatusCode::Ok)
+                .header(rtsp_types::headers::CONTENT_TYPE, "application/sdp")
+                .build(Bytes::from_static(include_bytes!(
+                    "testdata/geovision_sdp.txt"
+                )));
+        super::parse_describe(url, &response).unwrap();
+    }
+
     #[test]
     fn dahua_h264_aac_onvif() {
         // DESCRIBE.
@@ -692,7 +727,13 @@ mod tests {
         // SETUP.
         let setup_response = response(include_bytes!("testdata/dahua_setup.txt"));
         let setup_response = super::parse_setup(&setup_response).unwrap();
-        assert_eq!(setup_response.session_id, "634214675641");
+        assert_eq!(
+            setup_response.session,
+            SessionHeader {
+                id: "634214675641".into(),
+                timeout_sec: 60
+            }
+        );
         assert_eq!(setup_response.channel_id, Some(0));
         assert_eq!(setup_response.ssrc, Some(0x30a98ee7));
         p.streams[0].state = StreamState::Init(StreamStateInit {
@@ -789,7 +830,13 @@ mod tests {
         // SETUP.
         let setup_response = response(include_bytes!("testdata/hikvision_setup.txt"));
         let setup_response = super::parse_setup(&setup_response).unwrap();
-        assert_eq!(setup_response.session_id, "708345999");
+        assert_eq!(
+            setup_response.session,
+            SessionHeader {
+                id: "708345999".into(),
+                timeout_sec: 60
+            }
+        );
         assert_eq!(setup_response.channel_id, Some(0));
         assert_eq!(setup_response.ssrc, Some(0x4cacc3d1));
         p.streams[0].state = StreamState::Init(StreamStateInit {
@@ -825,6 +872,10 @@ mod tests {
         let base = "rtsp://192.168.5.206/h264Preview_01_main/";
         assert_eq!(p.control.as_str(), base);
         assert!(!p.accept_dynamic_rate);
+        assert_eq!(
+            p.tool.as_deref(),
+            Some("LIVE555 Streaming Media v2013.04.08")
+        );
 
         assert_eq!(p.streams.len(), 2);
 
@@ -864,7 +915,13 @@ mod tests {
         // SETUP.
         let setup_response = response(include_bytes!("testdata/reolink_setup.txt"));
         let setup_response = super::parse_setup(&setup_response).unwrap();
-        assert_eq!(setup_response.session_id, "F8F8E425");
+        assert_eq!(
+            setup_response.session,
+            SessionHeader {
+                id: "F8F8E425".into(),
+                timeout_sec: 60
+            }
+        );
         assert_eq!(setup_response.channel_id, Some(0));
         assert_eq!(setup_response.ssrc, None);
         p.streams[0].state = StreamState::Init(StreamStateInit::default());
@@ -942,7 +999,13 @@ mod tests {
         // SETUP.
         let setup_response = response(include_bytes!("testdata/bunny_setup.txt"));
         let setup_response = super::parse_setup(&setup_response).unwrap();
-        assert_eq!(setup_response.session_id, "1642021126");
+        assert_eq!(
+            setup_response.session,
+            SessionHeader {
+                id: "1642021126".into(),
+                timeout_sec: 60
+            }
+        );
         assert_eq!(setup_response.channel_id, Some(0));
         assert_eq!(setup_response.ssrc, None);
         p.streams[0].state = StreamState::Init(StreamStateInit::default());
@@ -967,6 +1030,10 @@ mod tests {
         let p = parse_describe(prefix, include_bytes!("testdata/foscam_describe.txt")).unwrap();
         assert_eq!(p.control.as_str(), &(prefix.to_string() + "/"));
         assert!(!p.accept_dynamic_rate);
+        assert_eq!(
+            p.tool.as_deref(),
+            Some("LIVE555 Streaming Media v2014.02.10")
+        );
 
         assert_eq!(p.streams.len(), 2);
 
@@ -1103,7 +1170,13 @@ mod tests {
         // SETUP.
         let setup_response = response(include_bytes!("testdata/gw_main_setup_video.txt"));
         let setup_response = super::parse_setup(&setup_response).unwrap();
-        assert_eq!(setup_response.session_id, "9a90de54");
+        assert_eq!(
+            setup_response.session,
+            SessionHeader {
+                id: "9a90de54".into(),
+                timeout_sec: 60
+            }
+        );
         assert_eq!(setup_response.channel_id, Some(0));
         assert_eq!(setup_response.ssrc, None);
         p.streams[0].state = StreamState::Init(StreamStateInit {
@@ -1114,7 +1187,13 @@ mod tests {
 
         let setup_response = response(include_bytes!("testdata/gw_main_setup_audio.txt"));
         let setup_response = super::parse_setup(&setup_response).unwrap();
-        assert_eq!(setup_response.session_id, "9a90de54");
+        assert_eq!(
+            setup_response.session,
+            SessionHeader {
+                id: "9a90de54".into(),
+                timeout_sec: 60
+            }
+        );
         assert_eq!(setup_response.channel_id, Some(2));
         assert_eq!(setup_response.ssrc, None);
         p.streams[1].state = StreamState::Init(StreamStateInit {
@@ -1183,7 +1262,13 @@ mod tests {
         // SETUP.
         let setup_response = response(include_bytes!("testdata/gw_sub_setup.txt"));
         let setup_response = super::parse_setup(&setup_response).unwrap();
-        assert_eq!(setup_response.session_id, "9b0d0e54");
+        assert_eq!(
+            setup_response.session,
+            SessionHeader {
+                id: "9b0d0e54".into(),
+                timeout_sec: 60
+            }
+        );
         assert_eq!(setup_response.channel_id, Some(0));
         assert_eq!(setup_response.ssrc, None);
         p.streams[0].state = StreamState::Init(StreamStateInit {
